@@ -54,7 +54,8 @@ static GList      *nautilus_do_component_query                   (const char
 								  gboolean                  ignore_content_mime_types,
 								  GList                    *explicit_iids,
 								  char                    **extra_sort_criteria,
-								  char                     *extra_requirements);
+								  char                     *extra_requirements,
+								  gboolean                  must_be_view);
 static GList      *str_list_difference                           (GList                    *a,
 								  GList                    *b);
 static char      **strv_concat                                   (char                    **a,
@@ -69,12 +70,11 @@ is_known_mime_type (const char *mime_type)
 static gboolean
 nautilus_mime_actions_check_if_minimum_attributes_ready (NautilusFile *file)
 {
-	GList *attributes;
+	NautilusFileAttributes attributes;
 	gboolean ready;
 
 	attributes = nautilus_mime_actions_get_minimum_file_attributes ();
 	ready = nautilus_file_check_if_ready (file, attributes);
-	g_list_free (attributes);
 
 	return ready;
 }
@@ -82,40 +82,32 @@ nautilus_mime_actions_check_if_minimum_attributes_ready (NautilusFile *file)
 static gboolean
 nautilus_mime_actions_check_if_full_attributes_ready (NautilusFile *file)
 {
-	GList *attributes;
+	NautilusFileAttributes attributes;
 	gboolean ready;
 
 	attributes = nautilus_mime_actions_get_full_file_attributes ();
 	ready = nautilus_file_check_if_ready (file, attributes);
-	g_list_free (attributes);
 
 	return ready;
 }
 
 
-GList *
+NautilusFileAttributes 
 nautilus_mime_actions_get_minimum_file_attributes (void)
 {
-	GList *attributes;
-
-	attributes = NULL;
-	attributes = g_list_prepend (attributes, NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI);
-	attributes = g_list_prepend (attributes, NAUTILUS_FILE_ATTRIBUTE_METADATA);
-	attributes = g_list_prepend (attributes, NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE);
-
-	return attributes;
+	return NAUTILUS_FILE_ATTRIBUTE_VOLUMES |
+		NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI |
+		NAUTILUS_FILE_ATTRIBUTE_METADATA |
+		NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE |
+		NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE;
 }
 
 
-GList *
+NautilusFileAttributes 
 nautilus_mime_actions_get_full_file_attributes (void)
 {
-	GList *attributes;
-
-	attributes = nautilus_mime_actions_get_minimum_file_attributes ();
-	attributes = g_list_prepend (attributes, NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES);
-
-	return attributes;
+	return nautilus_mime_actions_get_minimum_file_attributes () |
+		NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES;
 }
 
 
@@ -331,6 +323,7 @@ nautilus_mime_get_default_component_sort_conditions (NautilusFile *file, char *d
 
 static Bonobo_ServerInfo *
 nautilus_mime_get_default_component_for_file_internal (NautilusFile *file,
+						       gboolean      fallback,
 						       gboolean     *user_chosen)
 {
 	GList *info_list;
@@ -365,8 +358,11 @@ nautilus_mime_get_default_component_for_file_internal (NautilusFile *file,
 		item_mime_types = NULL;
 	}
 
-	default_component_string = nautilus_file_get_metadata 
-		(file, NAUTILUS_METADATA_KEY_DEFAULT_COMPONENT, NULL);
+	default_component_string = NULL;
+	if (!fallback) {
+		default_component_string = nautilus_file_get_metadata 
+			(file, NAUTILUS_METADATA_KEY_DEFAULT_COMPONENT, NULL);
+	} 
 
     	if (default_component_string == NULL) {
 		metadata_default = FALSE;
@@ -393,13 +389,13 @@ nautilus_mime_get_default_component_for_file_internal (NautilusFile *file,
 	if (metadata_default) {
 		extra_requirements = g_strconcat ("iid == '", default_component_string, "'", NULL);
 		info_list = nautilus_do_component_query (mime_type, uri_scheme, item_mime_types, TRUE,
-							 explicit_iids, sort_conditions, extra_requirements);
+							 explicit_iids, sort_conditions, extra_requirements, TRUE);
 		g_free (extra_requirements);
 	}
 
 	if (info_list == NULL) {
 		info_list = nautilus_do_component_query (mime_type, uri_scheme, item_mime_types, FALSE, 
-							 explicit_iids, sort_conditions, NULL);
+							 explicit_iids, sort_conditions, NULL, TRUE);
 	}
 
 	if (info_list != NULL) {
@@ -432,16 +428,23 @@ nautilus_mime_get_default_component_for_file_internal (NautilusFile *file,
 Bonobo_ServerInfo *
 nautilus_mime_get_default_component_for_file (NautilusFile      *file)
 {
-	return nautilus_mime_get_default_component_for_file_internal (file, NULL);
+	return nautilus_mime_get_default_component_for_file_internal (file, FALSE, NULL);
 }
 
+Bonobo_ServerInfo *
+nautilus_mime_get_default_fallback_component_for_file (NautilusFile      *file)
+{
+	return nautilus_mime_get_default_component_for_file_internal (file, TRUE, NULL);
+}
+
+
 gboolean
 nautilus_mime_is_default_component_for_file_user_chosen (NautilusFile      *file)
 {
 	Bonobo_ServerInfo *component;
 	gboolean user_chosen;
 
-	component = nautilus_mime_get_default_component_for_file_internal (file, &user_chosen);
+	component = nautilus_mime_get_default_component_for_file_internal (file, FALSE, &user_chosen);
 
 	/* Doesn't count as user chosen if the user-specified data is bogus and doesn't
 	 * result in an actual component.
@@ -455,7 +458,6 @@ nautilus_mime_is_default_component_for_file_user_chosen (NautilusFile      *file
 	return user_chosen;
 }
 
-
 GList *
 nautilus_mime_get_short_list_applications_for_file (NautilusFile      *file)
 {
@@ -521,6 +523,105 @@ nautilus_mime_get_short_list_applications_for_file (NautilusFile      *file)
 	return result;
 }
 
+static GList *
+get_open_with_mime_applications (NautilusFile *file)
+{
+	char *guessed_mime_type;
+	char *mime_type;
+	GList *result;
+
+	guessed_mime_type = nautilus_file_get_guessed_mime_type (file);
+	mime_type = nautilus_file_get_mime_type (file);
+
+	result = gnome_vfs_mime_get_short_list_applications (mime_type);
+
+	if (strcmp (guessed_mime_type, mime_type) != 0) {
+		GList *result_2;
+		GList *l;
+
+		result_2 = gnome_vfs_mime_get_short_list_applications (guessed_mime_type);
+		for (l = result_2; l != NULL; l = l->next) {
+			if (!g_list_find_custom (result,
+						 ((GnomeVFSMimeApplication*)l->data)->id,
+						 (GCompareFunc) gnome_vfs_mime_application_has_id)) {
+				result = g_list_prepend (result, l->data);
+			}
+		}
+		g_list_free (result_2);
+	}
+
+	g_free (mime_type);
+	g_free (guessed_mime_type);
+	
+	return result;
+}
+
+/* Get a list of applications for the Open With menu.  This is 
+ * different than nautilus_mime_get_short_list_applications_for_file()
+ * because this function will merge the lists of the fast and slow
+ * mime types for the file */
+GList *
+nautilus_mime_get_open_with_applications_for_file (NautilusFile      *file)
+{
+	char *uri_scheme;
+	GList *result;
+	GList *removed;
+	GList *metadata_application_add_ids;
+	GList *metadata_application_remove_ids;
+	GList *p;
+	GnomeVFSMimeApplication *application;
+
+	if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+		return NULL;
+	}
+
+	result = get_open_with_mime_applications (file);
+
+	/* First remove applications that cannot support this location */
+	uri_scheme = nautilus_file_get_uri_scheme (file);
+	g_assert (uri_scheme != NULL);
+	result = eel_g_list_partition (result, application_supports_uri_scheme,
+					    uri_scheme, &removed);
+	gnome_vfs_mime_application_list_free (removed);
+	g_free (uri_scheme);
+	
+	metadata_application_add_ids = nautilus_file_get_metadata_list 
+		(file,
+		 NAUTILUS_METADATA_KEY_SHORT_LIST_APPLICATION_ADD,
+		 NAUTILUS_METADATA_SUBKEY_APPLICATION_ID);
+	metadata_application_remove_ids = nautilus_file_get_metadata_list 
+		(file,
+		 NAUTILUS_METADATA_KEY_SHORT_LIST_APPLICATION_REMOVE,
+		 NAUTILUS_METADATA_SUBKEY_APPLICATION_ID);
+
+
+	result = eel_g_list_partition (result, (EelPredicateFunction) gnome_vfs_mime_application_has_id_not_in_list, 
+				       metadata_application_remove_ids, &removed);
+	
+	gnome_vfs_mime_application_list_free (removed);
+
+	result = g_list_reverse (result);
+	for (p = metadata_application_add_ids; p != NULL; p = p->next) {
+		if (g_list_find_custom (result,
+					p->data,
+					(GCompareFunc) gnome_vfs_mime_application_has_id) == NULL &&
+		    g_list_find_custom (metadata_application_remove_ids,
+					p->data,
+					(GCompareFunc) strcmp) == NULL) {
+			application = gnome_vfs_application_registry_get_mime_application (p->data);
+			if (application != NULL) {
+				result = g_list_prepend (result, application);
+			}
+		}
+	}
+	result = g_list_reverse (result);
+
+	eel_g_list_free_deep (metadata_application_add_ids);
+	eel_g_list_free_deep (metadata_application_remove_ids);
+
+	return result;
+}
+
 static char *
 build_joined_string (GList *list, const char *prefix, const char *separator, const char *suffix)
 {
@@ -624,7 +725,7 @@ nautilus_mime_get_short_list_components_for_file (NautilusFile *file)
 		extra_sort_conditions[1] = NULL;
 		extra_requirements = build_joined_string (iids, "has (['", "','", "'], iid)");
 		result = nautilus_do_component_query (mime_type, uri_scheme, item_mime_types, FALSE,
-						      explicit_iids, extra_sort_conditions, extra_requirements);
+						      explicit_iids, extra_sort_conditions, extra_requirements, TRUE);
 		g_free (extra_requirements);
 		g_free (extra_sort_conditions[0]);
 	}
@@ -749,7 +850,7 @@ nautilus_mime_actions_file_needs_full_file_attributes (NautilusFile *file)
 	explicit_iids = get_explicit_content_view_iids_from_metafile (file); 
 
 	info_list = nautilus_do_component_query (mime_type, uri_scheme, NULL, TRUE,
-						 explicit_iids, NULL, NULL);
+						 explicit_iids, NULL, NULL, TRUE);
 	
 	needs_full_attributes = FALSE;
 
@@ -793,7 +894,7 @@ nautilus_mime_get_all_components_for_file_extended (NautilusFile *file,
 	info_list = nautilus_do_component_query (mime_type, uri_scheme,
 						 item_mime_types, FALSE,
 						 explicit_iids, NULL,
-						 extra_reqs);
+						 extra_reqs, TRUE);
 	
 	eel_g_list_free_deep (explicit_iids);
 	eel_g_list_free_deep (item_mime_types);
@@ -804,6 +905,195 @@ nautilus_mime_get_all_components_for_file_extended (NautilusFile *file,
 	return info_list;
 }
 
+static NautilusFileAttributes 
+nautilus_mime_actions_get_popup_file_attributes (void)
+{
+	return NAUTILUS_FILE_ATTRIBUTE_VOLUMES |
+		NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI |
+		NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE;
+}
+
+static gboolean
+nautilus_mime_actions_check_if_popup_attributes_ready (NautilusFile *file)
+{
+	NautilusFileAttributes attributes;
+	gboolean ready;
+
+	attributes = nautilus_mime_actions_get_popup_file_attributes ();
+	ready = nautilus_file_check_if_ready (file, attributes);
+
+	return ready;
+}
+
+GList *
+nautilus_mime_get_popup_components_for_file (NautilusFile *file)
+{
+	char *mime_type;
+	char *uri_scheme;
+	char *extra_reqs;
+	GList *item_mime_types;
+	GList *info_list;
+
+	if (!nautilus_mime_actions_check_if_popup_attributes_ready (file)) {
+		return NULL;
+	}
+
+	uri_scheme = nautilus_file_get_uri_scheme (file);
+
+	mime_type = nautilus_file_get_mime_type (file);
+
+	if (!nautilus_mime_actions_check_if_full_attributes_ready (file) || 
+	    !nautilus_file_get_directory_item_mime_types (file, &item_mime_types)) {
+		item_mime_types = NULL;
+	}
+
+	extra_reqs = "repo_ids.has ('IDL:Bonobo/Listener:1.0') AND (nautilus:context_menu_handler == true) AND nautilus:can_handle_multiple_files.defined()";
+
+	info_list = nautilus_do_component_query (mime_type, uri_scheme,
+						 item_mime_types, FALSE,
+						 NULL, NULL,
+						 extra_reqs, FALSE);
+	
+	eel_g_list_free_deep (item_mime_types);
+
+	g_free (uri_scheme);
+	g_free (mime_type);
+
+	return info_list;
+}
+
+static gboolean
+has_server_info_in_list (GList *list, Bonobo_ServerInfo *info)
+{
+	for (; list; list = list->next) {
+		Bonobo_ServerInfo *tmp_info = list->data;
+
+		if (strcmp (tmp_info->iid, info->iid) == 0) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static GList *
+server_info_list_intersection (GList *a, GList *b)
+{
+	GList *result = NULL;
+
+	if (a == NULL || b == NULL) {
+		return NULL;
+	}
+
+	while (b) {
+		Bonobo_ServerInfo *info;
+		
+		info = (Bonobo_ServerInfo *)b->data;
+
+		if (has_server_info_in_list (a, info)) {
+			result = g_list_prepend (result,
+				   Bonobo_ServerInfo_duplicate (info));
+
+		}
+		
+		b = b->next;
+	}
+
+	return g_list_reverse (result);
+}
+
+GList *
+nautilus_mime_get_property_components_for_file (NautilusFile *file)
+{
+        char *mime_type;
+        char *uri_scheme;
+        char *extra_reqs;
+        GList *item_mime_types;
+        GList *info_list;
+
+        if (!nautilus_mime_actions_check_if_minimum_attributes_ready (file)) {
+                return NULL;
+        }
+
+        uri_scheme = nautilus_file_get_uri_scheme (file);
+
+        mime_type = nautilus_file_get_mime_type (file);
+
+        if (!nautilus_mime_actions_check_if_full_attributes_ready (file) ||
+            !nautilus_file_get_directory_item_mime_types (file, &item_mime_types
+)) {
+                item_mime_types = NULL;
+        }
+
+        extra_reqs = "repo_ids.has ('IDL:Bonobo/Control:1.0') AND nautilus:property_page_name.defined()";
+
+        info_list = nautilus_do_component_query (mime_type, uri_scheme,
+                                                 item_mime_types, FALSE,
+                                                 NULL, NULL,
+                                                 extra_reqs, FALSE);
+
+        eel_g_list_free_deep (item_mime_types);
+
+        g_free (uri_scheme);
+        g_free (mime_type);
+
+        return info_list;
+}     
+
+GList *
+nautilus_mime_get_property_components_for_files (GList *files)
+{
+	GList *result, *l;
+
+	result = NULL;
+
+	for (l = files; l; l = l->next) {
+		GList *components, *new_result;
+
+		components = nautilus_mime_get_property_components_for_file (l->data);
+		if (result != NULL) {
+			new_result = server_info_list_intersection (result,
+								    components);
+			gnome_vfs_mime_component_list_free (result);
+			gnome_vfs_mime_component_list_free (components);
+			result = new_result;
+		} else {
+			result = components;;
+		}
+
+
+	}	
+
+	return result;
+}
+
+GList *
+nautilus_mime_get_popup_components_for_files (GList *files)
+{
+	GList *result, *l;
+
+	result = NULL;
+
+	for (l = files; l; l = l->next) {
+		GList *components, *new_result;
+
+		components = nautilus_mime_get_popup_components_for_file (l->data);
+		if (result != NULL) {
+			new_result = server_info_list_intersection (result,
+								   components);
+			gnome_vfs_mime_component_list_free (result);
+			gnome_vfs_mime_component_list_free (components);
+			result = new_result;
+		} else {
+			result = components;;
+		}
+
+
+	}	
+
+	return result;
+}
+
 GList *
 nautilus_mime_get_all_components_for_file (NautilusFile *file)
 {
@@ -838,7 +1128,7 @@ mime_get_all_components_for_uri_scheme (const char *uri_scheme)
 
 	return nautilus_do_component_query
 		(NULL, uri_scheme, NULL, TRUE,
-		 NULL, NULL, NULL);
+		 NULL, NULL, NULL, TRUE);
 }
 
 gboolean
@@ -1312,32 +1602,28 @@ static char *
 make_bonobo_activation_query_with_known_mime_type (const char *mime_type, 
 				     const char *uri_scheme, 
 				     GList      *explicit_iids, 
-				     const char *extra_requirements)
+				     const char *extra_requirements,
+				     gboolean    must_be_view)
 {
         char *mime_supertype;
         char *result;
         char *explicit_iid_query;
+	const char *view_as_name_logic;
 
         mime_supertype = mime_type_get_supertype (mime_type);
 
         explicit_iid_query = make_bonobo_activation_query_for_explicit_content_view_iids (explicit_iids);
 
+	if (must_be_view) {
+		view_as_name_logic = "nautilus:view_as_name.defined ()";
+	} else {
+		view_as_name_logic = "true";
+	}
+
         result = g_strdup_printf 
                 (
-                 /* Check if the component has the interfaces we need.
-                  * We can work with either a Nautilus View, or
-                  * with a Bonobo Control or Embeddable that supports
-                  * one of the three persistence interfaces:
-                  * PersistStream, ProgressiveDataSink, or
-                  * PersistFile.
-                  */
-                 "(((repo_ids.has_all (['IDL:Bonobo/Control:1.0',"
-                                      "'IDL:Nautilus/View:1.0'])"
-                  "OR (repo_ids.has_one (['IDL:Bonobo/Control:1.0',"
-                                         "'IDL:Bonobo/Embeddable:1.0'])"
-                      "AND repo_ids.has_one (['IDL:Bonobo/PersistStream:1.0',"
-                                             "'IDL:Bonobo/ProgressiveDataSink:1.0',"
-                                             "'IDL:Bonobo/PersistFile:1.0'])))"
+
+                 
                  
                  /* Check that the component either has a specific
                   * MIME type or URI scheme. If neither is specified,
@@ -1345,7 +1631,7 @@ make_bonobo_activation_query_with_known_mime_type (const char *mime_type,
                   * and all schemes". For that, you have to do a
                   * wildcard for the MIME type or for the scheme.
                   */
-                 "AND (bonobo:supported_mime_types.defined ()"
+                 "(bonobo:supported_mime_types.defined ()"
                       "OR bonobo:supported_uri_schemes.defined ()"
 		      "OR bonobo:additional_uri_schemes.defined ())"
 
@@ -1379,12 +1665,13 @@ make_bonobo_activation_query_with_known_mime_type (const char *mime_type,
 		 "OR (bonobo:additional_uri_schemes.has ('%s')"
                       "OR bonobo:additional_uri_schemes.has ('*')))"
 
-                  /* Check that the component makes it clear that it's
+		 /* Check that the component makes it clear that it's
                    * intended for Nautilus by providing a "view_as"
                    * name. We could instead support a default, but
                    * that would make components that are untested with
                    * Nautilus appear.  */
-                 "AND nautilus:view_as_name.defined ())"
+		  "AND %s)"
+                  
 
                   /* Also select iids that were specifically requested
                      for this location, even if they do not otherwise
@@ -1397,14 +1684,42 @@ make_bonobo_activation_query_with_known_mime_type (const char *mime_type,
                  /* The MIME type, MIME supertype, and URI scheme for
                   * the %s above.
                   */
-                 , mime_type, mime_supertype, uri_scheme, uri_scheme
+                 , mime_type, mime_supertype, uri_scheme, uri_scheme,
 
                  /* The explicit metafile iid query for the %s above. */
-                 , explicit_iid_query
+                 view_as_name_logic, explicit_iid_query
 
 		 /* extra requirements */
 		 , extra_requirements != NULL ? extra_requirements : "true");
 
+	if (must_be_view) {
+		char *str;
+
+
+                 /* Check if the component has the interfaces we need.
+                  * We can work with either a Nautilus View, or
+                  * with a Bonobo Control or Embeddable that supports
+                  * one of the three persistence interfaces:
+                  * PersistStream, ProgressiveDataSink, or
+                  * PersistFile.
+                  */
+		str = g_strdup_printf ("(((repo_ids.has_all (['IDL:Bonobo/Control:1.0',"
+                                      "'IDL:Nautilus/View:1.0'])"
+                  "OR (repo_ids.has_one (['IDL:Bonobo/Control:1.0',"
+                                         "'IDL:Bonobo/Embeddable:1.0'])"
+                      "AND repo_ids.has_one (['IDL:Bonobo/PersistStream:1.0',"
+                                             "'IDL:Bonobo/ProgressiveDataSink:1.0',"
+                                          "'IDL:Bonobo/PersistFile:1.0']))) "
+					  "AND %s", result);
+		g_free (result);
+		result = str;
+	} else {
+		char *str;
+		str = g_strdup_printf ("((%s", result);
+		g_free (result);
+		result = str;
+	}
+
         g_free (mime_supertype);
         g_free (explicit_iid_query);
         return result;
@@ -1413,31 +1728,28 @@ make_bonobo_activation_query_with_known_mime_type (const char *mime_type,
 static char *
 make_bonobo_activation_query_with_uri_scheme_only (const char *uri_scheme, 
 				     GList      *explicit_iids, 
-				     const char *extra_requirements)
+				     const char *extra_requirements,
+				     gboolean    must_be_view)
 {
         char *result;
         char *explicit_iid_query;
+	const char *view_as_name_logic;
         
         explicit_iid_query = make_bonobo_activation_query_for_explicit_content_view_iids (explicit_iids);
 
+	if (must_be_view) {
+		view_as_name_logic = "nautilus:view_as_name.defined ()";
+	} else {
+		view_as_name_logic = "true";
+	}
+
         result = g_strdup_printf 
                 (
-                 /* Check if the component has the interfaces we need.
-                  * We can work with either a Nautilus tView, or
-                  * with a Bonobo Control or Embeddable that works on
-                  * a file, which is indicated by Bonobo PersistFile.
-                  */
-                  "(((repo_ids.has_all(['IDL:Bonobo/Control:1.0',"
-                                      "'IDL:Nautilus/View:1.0'])"
-                   "OR (repo_ids.has_one(['IDL:Bonobo/Control:1.0',"
-                                         "'IDL:Bonobo/Embeddable:1.0'])"
-                       "AND repo_ids.has('IDL:Bonobo/PersistFile:1.0')))"
-
 
                   /* Check if the component supports this particular
                    * URI scheme.
                    */
-                  "AND (((bonobo:supported_uri_schemes.has ('%s')"
+                  "(((bonobo:supported_uri_schemes.has ('%s')"
                          "OR bonobo:supported_uri_schemes.has ('*'))"
 
                   /* Check that the component doesn't require
@@ -1452,13 +1764,12 @@ make_bonobo_activation_query_with_uri_scheme_only (const char *uri_scheme,
 		  "OR (bonobo:additional_uri_schemes.has ('%s')"
 		      "OR bonobo:additional_uri_schemes.has ('*')))"
 
-
-                  /* Check that the component makes it clear that it's
+		 /* Check that the component makes it clear that it's
                    * intended for Nautilus by providing a "view_as"
                    * name. We could instead support a default, but
                    * that would make components that are untested with
                    * Nautilus appear.  */
-                  "AND nautilus:view_as_name.defined ())"
+		  "AND %s)"
 
                  /* Also select iids that were specifically requested
                      for this location, even if they do not otherwise
@@ -1470,11 +1781,41 @@ make_bonobo_activation_query_with_uri_scheme_only (const char *uri_scheme,
 		  " AND (%s)"
 
                   /* The URI scheme for the %s above. */
-                  , uri_scheme, uri_scheme
+                  , uri_scheme, uri_scheme, view_as_name_logic
 
                   /* The explicit metafile iid query for the %s above. */
                   , explicit_iid_query,
 		  extra_requirements != NULL ? extra_requirements : "true");
+	
+
+	if (must_be_view) {
+		char *str;
+
+
+                 /* Check if the component has the interfaces we need.
+                  * We can work with either a Nautilus View, or
+                  * with a Bonobo Control or Embeddable that supports
+                  * one of the three persistence interfaces:
+                  * PersistStream, ProgressiveDataSink, or
+                  * PersistFile.
+                  */
+		str = g_strdup_printf ("(((repo_ids.has_all (['IDL:Bonobo/Control:1.0',"
+                                      "'IDL:Nautilus/View:1.0'])"
+                  "OR (repo_ids.has_one (['IDL:Bonobo/Control:1.0',"
+                                         "'IDL:Bonobo/Embeddable:1.0'])"
+                      "AND repo_ids.has_one (['IDL:Bonobo/PersistStream:1.0',"
+                                             "'IDL:Bonobo/ProgressiveDataSink:1.0',"
+                                          "'IDL:Bonobo/PersistFile:1.0']))) "
+					  "AND %s", result);
+		g_free (result);
+		result = str;
+	} else {
+		char *str;
+		str = g_strdup_printf ("((%s", result);
+		g_free (result);
+		result = str;
+	}
+		  
 
 	g_free (explicit_iid_query);
 	
@@ -1591,7 +1932,8 @@ nautilus_do_component_query (const char        *mime_type,
 			     gboolean           ignore_content_mime_types,
 			     GList             *explicit_iids,
 			     char             **extra_sort_criteria,
-			     char              *extra_requirements)
+			     char              *extra_requirements,
+			     gboolean           must_be_view)
 { 
 	Bonobo_ServerInfoList *bonobo_activation_result;
 	char *query;
@@ -1603,9 +1945,9 @@ nautilus_do_component_query (const char        *mime_type,
         query = NULL;
 
         if (is_known_mime_type (mime_type)) {
-                query = make_bonobo_activation_query_with_known_mime_type (mime_type, uri_scheme, explicit_iids, extra_requirements);
+                query = make_bonobo_activation_query_with_known_mime_type (mime_type, uri_scheme, explicit_iids, extra_requirements, must_be_view);
         } else {
-                query = make_bonobo_activation_query_with_uri_scheme_only (uri_scheme, explicit_iids, extra_requirements);
+                query = make_bonobo_activation_query_with_uri_scheme_only (uri_scheme, explicit_iids, extra_requirements, must_be_view);
         }
 
 	all_sort_criteria = strv_concat (extra_sort_criteria, nautilus_sort_criteria);
